package pendo.io.pendo_sdk;

import android.app.Activity;
import android.app.Application;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.HashMap;
import java.util.Iterator;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import sdk.pendo.io.Pendo;

/** PendoFlutterPlugin */
public class PendoFlutterPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  final static String pendoChannelName = "pendo_sdk_channel_name";
  private static final String API_CALL_INIT_SDK              = "initSDK";
  private static final String API_CALL_CLEAR_VISITOR         = "clearVisitor";
  private static final String API_CALL_SWITCH_VISITOR        = "switchVisitor";
  private static final String API_CALL_SWITCH_VISITOR_DATA   = "setVisitorData";
  private static final String API_CALL_SET_ACCOUNT_DATA      = "setAccountData";
  private static final String API_CALL_INIT_WITHOUT_VISITOR  = "initSDKWithoutVisitor";
  private static final String API_CALL_TRACK                 = "track";
  private static final String API_CALL_END_SESSION           = "endSession";
  private static final String API_CALL_RESUME_GUIDES         = "resumeGuides";
  private static final String API_CALL_DISMISS_VISIBLE_GUIDE = "dismissVisibleGuides";
  private static final String API_CALL_SET_ACCOUNT_ID        = "setAccountId";
  private static final String API_CALL_PAUSE_GUIDES          = "pauseGuides";
  private static final String API_CALL_GET_VISITOR_ID        = "getVisitorId";
  private static final String API_CALL_GET_DEVICE_ID         = "getDeviceId";
  private static final String API_CALL_GET_ACCOUNT_ID        = "getAccountId";

  private static final String APP_KEY              = "ApplicationKey";
  private static final String APP_PARAM            = "ApplicationParameters";
  private static final String KEY_VISITOR_ID       = "VisitorIDKey";
  private static final String KEY_ACCOUNT_ID       = "AccountIDKey";
  private static final String KEY_VISITOR_DATA     = "VisitorDataKey";
  private static final String KEY_ACCOUNT_DATA     = "AccountDataKey";
  private static final String KEY_EVENT_NAME       = "EventNameKey";
  private static final String KEY_DISMISS_GUIDES   = "DismissGuidesKey";
  private static final String KEY_EVENT_PROPERTIES = "EventPropertiesKey";

  private static final String VAL_VISITOR_ID      = "visitorId";
  private static final String VAL_ACCOUNT_ID      = "accountId";
  private static final String VAL_VISITOR_DATA    = "visitorData";
  private static final String VAL_ACCOUNT_DATA    = "accountData";
  private static final String VAL_ENV_NAME        = "environmentName";

  

  private Activity mActivity;

  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  public PendoFlutterPlugin() {
    super();
  }
  private PendoFlutterPlugin(Activity activity) {
    super();
    this.mActivity = activity;
  }

  /**  FlutterPlugin **/
  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), pendoChannelName);
    channel.setMethodCallHandler(this);
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), pendoChannelName);
    channel.setMethodCallHandler(new PendoFlutterPlugin(registrar.activity()));
  }


  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  /**  ActivityAware **/
  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    this.mActivity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
    this.mActivity = null;
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    this.mActivity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivity() {
    this.mActivity = null;
  }

  /**  MethodCallHandler **/
  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case API_CALL_INIT_SDK:
        if (call.arguments instanceof HashMap) {
          HashMap arguments  = (HashMap) call.arguments;
          String  appKey     = (String ) arguments.get(APP_KEY);
          HashMap initParams = (HashMap) arguments.get(APP_PARAM);
          if (appKey != null) {
            initSDK(appKey, initParams);
          }
        }
        break;

      case API_CALL_CLEAR_VISITOR:
        clearVisitor();
        break;

      case API_CALL_SWITCH_VISITOR:
        if (call.arguments instanceof HashMap) {
          HashMap arguments = (HashMap) call.arguments;
          String    visitorId = (String) arguments.get(KEY_VISITOR_ID);
          String    accountId = (String) arguments.get(KEY_ACCOUNT_ID);
          HashMap visitorData = (HashMap) arguments.get(KEY_VISITOR_DATA);
          HashMap accountData = (HashMap) arguments.get(KEY_ACCOUNT_DATA);
          switchVisitor(visitorId,accountId,visitorData,accountData);
        }
        break;

      case API_CALL_SWITCH_VISITOR_DATA:
        if (call.arguments instanceof HashMap) {
          HashMap arguments = (HashMap) call.arguments;
          HashMap data      = (HashMap) arguments.get(KEY_VISITOR_DATA);
          setVisitorData(data);
        }
        break;

      case API_CALL_SET_ACCOUNT_DATA:
        if (call.arguments instanceof HashMap) {
          HashMap arguments = (HashMap) call.arguments;
          HashMap data      = (HashMap) arguments.get(KEY_ACCOUNT_DATA);
          setAccountData(data);
        }
        break;

      case API_CALL_INIT_WITHOUT_VISITOR:
        if (call.arguments instanceof HashMap) {
          HashMap arguments = (HashMap) call.arguments;
          String appKey = (String) arguments.get(APP_KEY);
          HashMap appParams = (HashMap) arguments.get(APP_PARAM);
          initSdkWithoutVisitor(appKey, appParams);
        }
        break;

     
      case API_CALL_TRACK:
        if (call.arguments instanceof HashMap) {
          HashMap arguments = (HashMap) call.arguments;
          String   eventName = (String) arguments.get(KEY_EVENT_NAME);
          HashMap properties = (HashMap) arguments.get(KEY_EVENT_PROPERTIES);
          track(eventName, properties);
        }
        break;

      case API_CALL_END_SESSION:
        endSession();
        break;

      case API_CALL_RESUME_GUIDES:
        resumeGuides();
        break;

      case API_CALL_DISMISS_VISIBLE_GUIDE:
        dismissVisibleGuides();
        break;

      case API_CALL_SET_ACCOUNT_ID:
        if (call.arguments instanceof HashMap) {
          HashMap arguments = (HashMap) call.arguments;
          String accountId = (String) arguments.get(KEY_ACCOUNT_ID);
          setAccountId(accountId);
        }
        break;
        
      case API_CALL_PAUSE_GUIDES:
        if (call.arguments instanceof HashMap) {
          HashMap arguments = (HashMap) call.arguments;
          boolean dismissGuides = (boolean) arguments.get(KEY_DISMISS_GUIDES);
          pauseGuides(dismissGuides);
        }
        break;

      case API_CALL_GET_VISITOR_ID:
          result.success(getVisitorId());
        break;

      case API_CALL_GET_ACCOUNT_ID:
        result.success(getAccountId());
        break;

      case API_CALL_GET_DEVICE_ID:
        result.success(getDeviceId());
        break;

      default:
        result.notImplemented();
        break;
    }
  }

  // public static synchronized void initSDK(Application app,String appKey, PendoInitParams params) {
  private void initSDK(@NonNull String appKey, @Nullable HashMap params) {
    Pendo.PendoInitParams initParams = new Pendo.PendoInitParams();
    initParams.setPendoOptions(new Pendo.PendoOptions.Builder().setIgnoreFirstOnCreate(true).build());
    if (params != null) {
      if (params.containsKey(VAL_VISITOR_ID)) {
        initParams.setVisitorId((String) params.get(VAL_VISITOR_ID));
      }
      if (params.containsKey(VAL_ACCOUNT_ID)) {
        initParams.setAccountId((String) params.get(VAL_ACCOUNT_ID));
      }
      if (params.containsKey(VAL_VISITOR_DATA)) {
        initParams.setVisitorData((HashMap<String, Object>)params.get(VAL_VISITOR_DATA));
      }
      if (params.containsKey(VAL_ACCOUNT_DATA)) {
        initParams.setAccountData((HashMap<String, Object>)params.get(VAL_ACCOUNT_DATA));
      }
      if (params.containsKey(VAL_ENV_NAME)) {
        String environmentName = (String)params.get(VAL_ENV_NAME);
        if (environmentName != "") {
          initParams.setPendoOptions(new Pendo.PendoOptions.Builder().setEnvironmentDebugOnly(environmentName).build());
        }
      }
    }
    Pendo.initSDK(this.mActivity, appKey, initParams);   // call initSDK with initParams as a third parameter (this can be `null`).
  }

  private void clearVisitor() {
    Pendo.clearVisitor();
  }

  private void switchVisitor(String visitorId, String accountId, @Nullable HashMap visitorData, @Nullable HashMap accountData) {
    Pendo.switchVisitor(visitorId, accountId, visitorData, accountData);
  }

  private void setVisitorData(@Nullable HashMap visitorData) {
    Pendo.setVisitorData(visitorData);
  }

  private void setAccountData(@Nullable HashMap accountData) {
    Pendo.setAccountData(accountData);
  }

  private void track(@NonNull String event, @Nullable HashMap properties) {
    if (isNullOrWhiteSpace(event)) {
      throw new IllegalArgumentException("Event name is required");
    }
    Pendo.track(event, properties);
  }

  private void initSdkWithoutVisitor(@NonNull String appKey, @Nullable HashMap appParams) {
    Pendo.PendoOptions.Builder pendoOptions =  new Pendo.PendoOptions.Builder().setIgnoreFirstOnCreate(true);
    if (appParams.containsKey(VAL_ENV_NAME)) {
      String environmentName = (String)appParams.get(VAL_ENV_NAME);
      if (!isNullOrWhiteSpace(environmentName)) {
        pendoOptions.setEnvironmentDebugOnly(environmentName);
      }
    }
    Pendo.initSdkWithoutVisitor(this.mActivity, appKey, pendoOptions.build());
  }

  private void endSession() {
    Pendo.endSession();
  }

  private void resumeGuides() {
    Pendo.resumeGuides();
  }

  private void dismissVisibleGuides() {
    Pendo.dismissVisibleGuides();
  }

  private void setAccountId(String accountId) {
    Pendo.setAccountId(accountId);
  }

  private void pauseGuides(boolean dismissGuides) {
    Pendo.pauseGuides(dismissGuides);
  }

  private String getVisitorId() {
    return Pendo.getVisitorId();
  }

  private String getAccountId() {
    return Pendo.getAccountId();
  }

  private String getDeviceId() {
    return Pendo.getDeviceId();
  }

  private static boolean isNullOrWhiteSpace(String value) {
    return value == null || value.trim().isEmpty();
  }
}
